@@ -2,7 +2,7 @@
  * Classes for greedy estimation of causal structures
  *
  * @author Alain Hauser
- * $Id: greedy.hpp 256 2014-04-09 11:54:39Z alhauser $
+ * $Id: greedy.hpp 393 2016-08-20 09:43:47Z alhauser $
  */
 
 #ifndef GREEDY_HPP_
@@ -20,6 +20,9 @@
 
 enum edge_flag { NOT_PROTECTED, UNDECIDABLE, PROTECTED };
 
+// Types of adaptiveness (cf. "ARGES")
+enum ForwardAdaptiveFlag { NONE, VSTRUCTURES, TRIPLES };
+
 /**
  * Help functions for easier handling of set operations
  */
@@ -84,7 +87,7 @@ struct EdgeCmp : public std::binary_function<Edge, Edge, bool>
 };
 
 /**
- * Helper class used as a stack for candidate cliques C \subset N
+ * Auxiliary class used as a stack for candidate cliques C \subset N
  */
 class CliqueStack : public std::deque<std::set<uint> >
 {
@@ -110,7 +113,7 @@ public:
 };
 
 /**
- * Helper classes for storing cached values
+ * Auxiliary classes for storing cached values
  */
 struct ArrowChange
 {
@@ -129,6 +132,112 @@ struct ArrowChangeCmp : public std::binary_function<Edge, Edge, bool>
 
 enum step_dir { SD_NONE, SD_FORWARD, SD_BACKWARD, SD_TURNING };
 
+/**
+ * Graph operations that can be logged
+ */
+enum graph_op { GO_ADD_EDGE, GO_REMOVE_EDGE, GO_LOCAL_SCORE };
+
+/**
+ * Auxiliary class for logging graph operations.
+ *
+ * This is a virtual base class that does not actually log operations;
+ * derived classes have to do that.
+ */
+class GraphOperationLogger
+{
+public:
+	/**
+	 * Constructor. Does nothing for base class.
+	 */
+	GraphOperationLogger() {};
+
+	/**
+	 * Destructor. Needs to be virtual because of different storage
+	 * of data in derived classes.
+	 */
+	virtual ~GraphOperationLogger() {};
+
+	/**
+	 * Reset logger
+	 */
+	virtual void reset() {};
+
+	/**
+	 * Log graph operation. If a single vertex is involved, it is specified
+	 * as "first vertex". If an edge is involved, source and target are specified
+	 * as first and second vertex, resp.
+	 */
+	virtual void log(graph_op operation, uint first, uint second = 0) {};
+};
+
+class EdgeOperationLogger : public GraphOperationLogger
+{
+protected:
+	/**
+	 * Sets of added and removed edges
+	 */
+	std::set<Edge, EdgeCmp> _addedEdges;
+	std::set<Edge, EdgeCmp> _removedEdges;
+
+public:
+	/**
+	 * Constructor
+	 */
+	EdgeOperationLogger() :
+		GraphOperationLogger(),
+		_addedEdges(),
+		_removedEdges() {};
+
+	/**
+	 * Destructor
+	 */
+	virtual ~EdgeOperationLogger() {};
+
+	/**
+	 * Reference to added or removed edges
+	 */
+	const std::set<Edge, EdgeCmp>& addedEdges() { return _addedEdges; }
+	const std::set<Edge, EdgeCmp>& removedEdges() { return _removedEdges; }
+
+	/**
+	 * Reset logger
+	 */
+	virtual void reset()
+	{
+		_addedEdges.clear();
+		_removedEdges.clear();
+	}
+
+	/**
+	 * Log edge additions or removals
+	 */
+	virtual void log(graph_op operation, uint first, uint second = 0)
+	{
+		Edge edge(first, second);
+		switch (operation) {
+			case GO_ADD_EDGE :
+				// If edge was already removed before, clear removal entry;
+				// otherwise add addition entry.
+				if (_removedEdges.erase(edge) == 0) {
+					_addedEdges.insert(edge);
+				}
+				break;
+
+			case GO_REMOVE_EDGE :
+				// If edge was already added before, clear addition entry;
+				// otherwise add removal entry.
+				if (_addedEdges.erase(edge) == 0) {
+					_removedEdges.insert(edge);
+				}
+				break;
+
+			default :
+				break;
+		}
+	}
+};
+
+
 // Forward declaration for testing
 class EssentialGraphTest;
 class BICScoreTest;
@@ -208,11 +317,21 @@ protected:
 	 */
 	boost::dynamic_bitset<> _childrenOnly; 
 
+	/**
+	 * Loggers for graph operations
+	 */
+	std::set<GraphOperationLogger*> _loggers;
+
 	/**
 	 * Checks whether there is a fixed gap between two vertices.
 	 */
 	bool gapFixed(const uint a, const uint b) const;
 
+	/**
+	 * Marks a gap as fixed or not fixed
+	 */
+	void setFixedGap(const uint a, const uint b, const bool fixed);
+
 	/**
 	 * Checks whether there is a path from a to b in the graph that does not
 	 * go through the vertices of C. The edge (a, b) is not considered, if it
@@ -506,6 +625,12 @@ public:
 	 */
 	std::set<uint> getChainComponent(const uint v) const;
 
+	/**
+	 * Adds and removes loggers. Functions return true on success.
+	 */
+	bool addLogger(GraphOperationLogger* logger);
+	bool removeLogger(GraphOperationLogger* logger);
+
 	/**
 	 * Sets and gets score object
 	 */
@@ -577,8 +702,11 @@ public:
 
 	/**
 	 * Does one forward step of the greedy interventional equivalence search.
+	 *
+	 * @param  adaptive: indicates whether set of allowed edges should be
+	 * adaptively enlarged according to AGES
 	 */
-	bool greedyForward();
+	bool greedyForward(const ForwardAdaptiveFlag adaptive = NONE);
 
 	/**
 	 * Does one backward step of the greedy interventional equivalence search
@@ -590,6 +718,12 @@ public:
 	 */
 	bool greedyTurn();
 
+	/**
+	 * Wrapper function to the greedy... functions; first argument indicates requested
+	 * direction
+	 */
+	bool greedyStepDir(const step_dir direction, const ForwardAdaptiveFlag adaptive = NONE);
+
 	/**
 	 * Does one greedy step, either forward, backward, or turning, the one that
 	 * yields the highest score gain.
@@ -614,6 +748,11 @@ public:
 	 */
 	bool greedyDAGTurn();
 
+	/**
+	 * Wrapper function for any of the three preceding functions
+	 */
+	bool greedyDAGStepDir(const step_dir direction);
+
 	/**
 	 * Maximizes the BIC score by dynamic programming, as proposed by
 	 * Silander and Myllymäki (2006). Only works for small graphs
@@ -633,4 +772,15 @@ public:
 	std::set<uint> getOptimalTarget(uint maxSize);
 };
 
+/**
+ * Reads in a graph from a list of in-edges passed as a SEXP to
+ * an EssentialGraph object
+ */
+EssentialGraph castGraph(const SEXP argInEdges);
+
+/**
+ * Wrap a graph structure to an R list of in-edges
+ */
+Rcpp::List wrapGraph(const EssentialGraph& graph);
+
 #endif /* GREEDY_HPP_ */
